from __future__ import print_function

import httplib2
import os
import os.path

import base64
import email

import datetime


from apiclient import discovery
from apiclient import errors
import oauth2client
from oauth2client import client
from oauth2client import tools

from HTMLParser import HTMLParser
from re import sub
from sys import stderr
from traceback import print_exc

# try:
#     import argparse
#     flags = argparse.ArgumentParser(parents=[tools.argparser]).parse_args()
# except ImportError:
#     flags = None

SCOPES = 'https://www.googleapis.com/auth/gmail.readonly'
CLIENT_SECRET_FILE = 'client_secret.json'
APPLICATION_NAME = 'Gmail API Python Quickstart'

def get_credentials():
	"""Gets valid user credentials from storage.

	If nothing has been stored, or if the stored credentials are invalid,
	the OAuth2 flow is completed to obtain the new credentials.

	Returns:
	Credentials, the obtained credential.
	"""
	home_dir = os.path.expanduser('~')
	credential_dir = os.path.join(home_dir, '.credentials')
	if not os.path.exists(credential_dir):
		os.makedirs(credential_dir)
	credential_path = os.path.join(credential_dir, 'gmail-python-quickstart.json')

	store = oauth2client.file.Storage(credential_path)
	credentials = store.get()
	if not credentials or credentials.invalid:
		flow = client.flow_from_clientsecrets(CLIENT_SECRET_FILE, SCOPES)
		flow.user_agent = APPLICATION_NAME
		# if flags:
		# 	credentials = tools.run_flow(flow, store, flags)
		# else: # Needed only for compatibility with Python 2.6
		credentials = tools.run(flow, store)
		print('Storing credentials to ' + credential_path)
	return credentials

def ListMessagesMatchingQuery(service, user_id, query=''):
    """List all Messages of the user's mailbox matching the query.

    Args:
    service: Authorized Gmail API service instance.
    user_id: User's email address. The special value "me"
    can be used to indicate the authenticated user.
    query: String used to filter messages returned.
    Eg.- 'from:user@some_domain.com' for Messages from a particular sender.

    Returns:
    List of Messages that match the criteria of the query. Note that the
    returned list contains Message IDs, you must use get with the
    appropriate ID to get the details of a Message.
    """
    try:
        response = service.users().messages().list(userId=user_id, q=query).execute()
        messages = []
        if 'messages' in response:
            messages.extend(response['messages'])

        while 'nextPageToken' in response:
            page_token = response['nextPageToken']
            response = service.users().messages().list(userId=user_id, q=query, pageToken=page_token).execute()
            messages.extend(response['messages'])

        return messages
    except errors.HttpError, error:
        print ('An error occurred: %s' % error)

def GetMessage(service, user_id, msg_id):
  """Get a Message with given ID.

  Args:
    service: Authorized Gmail API service instance.
    user_id: User's email address. The special value "me"
    can be used to indicate the authenticated user.
    msg_id: The ID of the Message required.

  Returns:
    A Message.
  """
  try:
    message = service.users().messages().get(userId=user_id, id=msg_id, format="raw").execute()

    # print ('Message snippet: %s' % message['snippet'])
    return message
  except errors.HttpError, error:
    print ('An error occurred: %s' % error)

#returns the datetime a certain message ID was sent
def getDate(service, msgID):
	#get internalDate in epoch ms:
	date = GetMessage(service, "me", msgID)['internalDate']

	#convert to float
	fdate = float(date)

	#convert to timestamp and then datetime
	return (datetime.datetime.fromtimestamp(fdate/1000.0))

#returns the number of days that passed since given date
def daysSince(date):
	days = date - datetime.datetime.today()
	sdays = str(days).split()

	#take abs value of day value (must convert from str -> int -> str)
	sdays[0] = str(abs(int(sdays[0])))

	if (sdays[0] == "1"):
		return (sdays[0] + " day")
	else:
		return (sdays[0] + " days")

#returns ID of latest email from specific email address
def getLatest(service, user_id, query=''):
  try:
    response = service.users().messages().list(userId=user_id,
                                               q=query, maxResults=1).execute()
    return(response['messages'][0])

    # if 'message' in response:
    # 	message.extend(response['message'])

    # messages = []
    # if 'messages' in response:
    #   messages.extend(response['messages'])

    # while 'nextPageToken' in response:
    #   page_token = response['nextPageToken']
    #   response = service.users().messages().list(userId=user_id, q=query,
    #                                      pageToken=page_token).execute()
    #   messages.extend(response['messages'])

    # return messages[0]
  except errors.HttpError, error:
    print ('An error occurred: %s' % error)

#returns text body parsed out of 'full' message from g.GetMessage()
def extractBody(rawMessage):
	msg_str = base64.urlsafe_b64decode(rawMessage['raw'].encode('UTF-8'))
	msg = email.message_from_string(msg_str)	
	for part in msg.walk():
		msg.get_payload()
		if part.get_content_type() == 'text/plain':
			# mytext = base64.urlsafe_b64decode(part.get_payload().encode('UTF-8'))
			mytext = part.get_payload()
			# print part.get_payload()
			return (mytext)	

class _DeHTMLParser(HTMLParser):
	def __init__(self):
		HTMLParser.__init__(self)
		self.__text = []

	def handle_data(self, data):
		text = data.strip()
		if len(text) > 0:
			text = sub('[ \t\r\n]+', ' ', text)
			self.__text.append(text + ' ')

	def handle_starttag(self, tag, attrs):
		if tag == 'p':
			self.__text.append('\n\n')
		elif tag == 'br':
			self.__text.append('\n')

	def handle_startendtag(self, tag, attrs):
		if tag == 'br':
			self.__text.append('\n\n')

	def text(self):
		return ''.join(self.__text).strip()

def dehtml(text):
	try:
		parser = _DeHTMLParser()
		parser.feed(text)
		parser.close()
		return parser.text()
	except:
		print_exc(file=stderr)
		return text


'''
Koto - Communication tracking CLI
Created by Ben Shukman

Usage:
	koto [-h | --help | --version]
	koto add <firstname> <lastname> [<email>]
	koto delete <firstname> [<lastname>]
	koto status (all | <firstname> [<lastname>])
	koto list [-e | --email] [-t | --time]
	koto import
	koto init

Optional arguments:
	-h --help   Show help dialog
	--version   Show verison number
	-e --email  Show emails
	-t --time   Show last time contacted

Koto commands:
	add        Add a new contact (only name is required)
	delete     Delete a contact
	status     Show a history of communications with an individual (or all)
	list 	  List the contacts in your database
	import     Import a CSV file of new contacts
	init       Set up koto and change settings

Planned features:
	koto commit <firstname> [<lastname>] <commit>
	Comprehensive koto status w/ commit tree branches
	Day -> month parser w/ hour support
'''


#koto - communication tracking utiliity
#created by ben shukman

#each name has attributes: last date contacted, each contact is a commit (so its a unique data struct)

#store JSON / class of data

#import list of names
#if recent email, display in high priority
#if name is under X days old, needs love

#TODO: add email validation and error handling for gmail errors
#TODO: add to database: validated field + last time communicated field
#TODO: koto status -> more comprehensive overview of comms
	#TODO: # of times talked to an investor
#TODO: finish main koto command
	# - run validation + update last time communicated (+ num. of times communicated if needed)
	# - 
#TODO: import file CSV

#write up read.me -> show ppl

#TODO: add commits
#TODO: add commit trees
#TODO: add person types, koto add [friend | associate | investor] 
#TODO: add commit types [-fb| -e| -m] w/ documentation
#TODO: for 'jeeves?' pull from sublime todo also
#format:
	# First Last - 6 days ago (if > 31 days, say months (1 month, 5 days ago))


import db_methods, gmail_methods

from docopt import docopt

g = gmail_methods
db = db_methods
versionNumber = '0.2.0'

def idGen(name, date):
	#generate id name such as 8839GODZILLA040494
	#use id's for commits
	pass

def main():
	arguments = (docopt(__doc__, version=versionNumber))

	#check if version is up to date

	if (db.firstStartupCheck()):
		if arguments['init']:
			pass
		else:
			print("This looks like your first time using koto.  Run 'koto init' to initialize setup.")
			db.exit()

	if arguments['init']:

		print('\nInitializing koto setup\n')

		import json
		#check if version up to date

		#gmail authentication
		print("Authenticating gmail connection")
		success = g.get_credentials()
		if (success):
			print("Authentication successful")

		#database creation
		db.initializeDB()

		#settings file creation
		db.initializeSettings()
		with open('/usr/local/Library/Koto/koto_settings.txt', 'w') as outfile:
			json.dump({'updateTime':'', 'responseTime':'',}, outfile)

		#set up personal usage settings
		print("\nHow often do you want to update your contacts? Answer in days")
		updateTime = db.selectNumber()
		#write updateTime to a file

		print("What is the maximum number of days before an email response is overdue?")
		responseTime = db.selectNumber()
		#write responseTime to a file

		with open('/usr/local/Library/Koto/koto_settings.txt', 'w') as outfile:
			json.dump({'updateTime': updateTime, 'responseTime': responseTime}, outfile)

		#ask to import CSV
		print("Do you have a list of contacts to import? (Y/N)")
		importBoolean = db.selectBoolean()
		if (importBoolean):
			db.importCSV()

		print("\nWelcome to koto!  Use '--help' to learn more")
		db.exit()

	credentials = g.get_credentials()
	http = credentials.authorize(g.httplib2.Http())
	service = g.discovery.build('gmail', 'v1', http=http)

	#grab updateTime and responseTime from a file

	if arguments['add']:
		if arguments['<email>']:
			# print("Adding " + str(arguments['<firstname>']) + " " + str(arguments['<lastname>']) + " to database")
			db.insertDB(arguments['<firstname>'].capitalize(), arguments['<lastname>'].capitalize())
			db.addEmail(arguments['<email>'], arguments['<firstname>'].capitalize(), arguments['<lastname>'].capitalize())
		else:
			# print("Adding " + str(arguments['<firstname>']) + " " + str(arguments['<lastname>']) + " to database")
			db.insertDB(arguments['<firstname>'].capitalize(), arguments['<lastname>'].capitalize())

	elif arguments['import']:
		db.importCSV()

	elif arguments['status']:
		if (arguments['all']):
			#grab email of each
			emails = db.readAllEmails()

			#look up ID of latest email

			print("Gathering last messages from database \n")
				
			for x in emails:

				name = db.readName(str(x[0]))
				firstName = name[0]
				lastName = name[1]

				#add owner name of email
				print("From: " + str(firstName) + " " + str(lastName) + " ({0})").format(x[0])
				msgID = (g.getLatest(service, "me", "from: {0}".format(x[0]))['id'])
				print("Message ID: " + msgID)

				#calculate date of email + days since last comm + get snippet

			 	#list the data
				date = g.getDate(service, msgID)
				print("Received: " + str(date))
				print(g.daysSince(date) + " since last communication")

				print("")
				print(g.GetMessage(service, "me", msgID)['snippet'])
				print("")



		else:
			print("Gathering last messages from database \n")

			matches = db.countMatches(arguments['<firstname>'].capitalize())

			#if multiple people exist with given first name
			if (matches > 1):

				lastName = db.selectPerson(arguments['<firstname>'].capitalize())

				emailID = db.readEmail(arguments['<firstname>'].capitalize(), lastName.capitalize())[0]
				print("") #to make space away from select person input
				print("From: " + arguments['<firstname>'].capitalize() + " " + lastName.capitalize() + " (" + emailID + ")")
				msgID = (g.getLatest(service, "me", "from: {0}".format(emailID))['id'])
				print("Message ID: " + msgID)

				#calculate date of email + days since last comm + get snippet

			 	#list the data
				date = g.getDate(service, msgID)
				print("Received: " + str(date))
				print(g.daysSince(date) + " since last communication")

				print("")
				print(g.GetMessage(service, "me", msgID)['snippet'])
				print("")

			#if only one person exists with given first name
			elif (matches == 1):
				lastName = db.readLastName(arguments['<firstname>'].capitalize())
				emailID = db.readEmail(arguments['<firstname>'].capitalize(), lastName)[0]
				print("From: " + arguments['<firstname>'].capitalize() + " " + str(lastName) + " (" + emailID + ")")
				msgID = (g.getLatest(service, "me", "from: {0}".format(emailID))['id'])
				print("Message ID: " + msgID)

				#calculate date of email + days since last comm + get snippet

			 	#list the data
				date = g.getDate(service, msgID)
				print("Received: " + str(date))
				print(g.daysSince(date) + " since last communication")

				print("")
				print(g.GetMessage(service, "me", msgID)['snippet'])
				print("")

			elif (matches == 0):
				print ('Error: person with that name does not exist')

	elif arguments['delete']:
		# print("Deleting " + str(arguments['<firstname>']) + " " + str(arguments['<lastname>']) + " from database \n")
		if (arguments['<lastname>']):
			db.deleteDB(arguments['<firstname>'].capitalize(), arguments['<lastname>'].capitalize())
		else:
			db.deleteDB(arguments['<firstname>'].capitalize())

	elif arguments['list']:

		names = db.allNames()

		for x in names:
			if ((arguments['-t'] | arguments['--time']) & (arguments['-e'] | arguments['--email'])):
				msgID = (g.getLatest(service, "me", "from: {0}".format(x[0] + ' ' + x[1]))['id'])
				date = g.getDate(service, msgID)
				days = g.daysSince(date)
				email = db.readEmail(x[0], x[1])[0]
				#month calculation code, create better version that grabs actual 28-31 day values later:
				# if (days >= 30):
				# 	while (days > 0):
				# 		days - 30
				# 		months += 1
				print(x[0] + ' ' + x[1] + ' (' + str(email) + ') - ' + days + ' ago')
			elif (arguments['-e'] | arguments['--email']):
				email = db.readEmail(x[0], x[1])[0]
				print(x[0] + ' ' + x[1] + ' (' + str(email) + ')')
			elif ((arguments['-t'] | arguments['--time'])):
				msgID = (g.getLatest(service, "me", "from: {0}".format(x[1]))['id'])
				date = g.getDate(service, msgID)
				days = g.daysSince(date)
				print(x[0] + ' ' + x[1] + ' - ' + days + ' ago')
			else:
				print(x[0] + ' ' + x[1])
				
	#standard koto command:
	else:
		# print('Gathering data...')
		import json

		needsLove = []
		highPriority = []
		new = []

		with open('/usr/local/Library/Koto/koto_settings.txt', 'r') as datafile:
			data = json.load(datafile)
			updateTime = data['updateTime']
			responseTime = data['responseTime']

		for name in db.allNames():
			msgID = (g.getLatest(service, "me", "from: {0}".format(name[0] + ' ' + name[1]))['id'])
			date = g.getDate(service, msgID)
			days = g.daysSince(date)
			if (int(days.split()[0]) <= 1):
				new.append('{0}'.format('\t' + name[0] + ' ' + name[1] + ' - ' + days + ' ago'))
			if (int(days.split()[0]) >= updateTime):
				needsLove.append('{0}'.format('\t' + name[0] + ' ' + name[1] + ' - ' + days + ' ago'))
			if ((1 < int(days.split()[0])) & (int(days.split()[0]) <= responseTime)):
				highPriority.append('{0}'.format('\t' + name[0] + ' ' + name[1] + ' - ' + days + ' ago'))

		print('\nNew messages:')
		print('=============')
		for x in new:
			print (x)

		print('\nOverdue')
		print('=======')
		for x in highPriority:
			print (x)

		if (len(needsLove) > 0):
			print('\nNeeds love:')
			print('===========')
			for x in needsLove:
				print (x)

		print('')

if __name__ == '__main__':
	main()




import sqlite3
import os
import sys

db_directory = '/usr/local/Library/Koto'
db_path = '/usr/local/Library/Koto/kotodb'

def firstStartupCheck():
	#if the database file exists
	if os.path.exists(db_path):
		#it is not the first time running koto
		return False
	else:
		return True

def initializeDB():
	print ('Initializing database in ' + db_path)
	db_directory = os.path.dirname(db_path)
	if not os.path.exists(db_directory):
		os.makedirs(db_directory)
		print ('Database directory created')
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("CREATE TABLE IF NOT EXISTS people(firstName text, lastName text, type text, email text, UNIQUE(firstName, lastName))")
	print('Database created')
	conn.close()

def initializeSettings():
	if not os.path.exists('/usr/local/Library/Koto/koto_settings.txt'):
		file = open('/usr/local/Library/Koto/koto_settings.txt', 'w')
		file.close()
	if (os.path.exists('/usr/local/Library/Koto/koto_settings.txt')):
		print('Settings file created')

def insertDB(firstName, lastName):
	print ('Inserting ' + firstName + ' ' + lastName + ' into database ' + db_path + '...')
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	# c.execute("CREATE TABLE IF NOT EXISTS people(firstName, lastName, UNIQUE(firstName, lastName))")
	c.executemany("INSERT OR IGNORE INTO people(firstName, lastName) VALUES (?, ?)", [(firstName, lastName)])
	if (c.rowcount != 0):
		print ('Successfully added ' + firstName + ' ' + lastName)
	else:
		print ('Failed to add ' + firstName + ' ' + lastName)
		#add already exists if statement error message
	conn.commit()
	conn.close()

def readAllEmails():
	print ('Reading contact emails from database ' + db_path + '...' + '\n')
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT email FROM people")
	return c.fetchall()
	conn.close()

def allNames():
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT firstName, lastName FROM people")
	return c.fetchall()
	conn.close()

def readDB(firstName):
	print ('Reading ' + firstName + ' from database ' + db_path + '...')
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT * FROM people WHERE firstName =?", [firstName])
	print (c.fetchall())
	conn.close()

def deleteDB(firstName, lastName=None):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	if (lastName):
		c.execute("DELETE FROM people WHERE firstName=? AND lastName=?", [firstName, lastName])
		if (c.rowcount != 0):
			print ('Successfully deleted ' + str(firstName) + ' ' + str(lastName))
		else:
			print ('Could not delete ' + str(firstName) + ' ' + str(lastName))
	else:

		matches = countMatches(firstName.capitalize())

		#if multiple people exist with given first name
		if (matches > 1):
			lastName = selectPerson(firstName)
			c.execute("DELETE FROM people WHERE firstName=? AND lastName=?", [firstName, lastName])
		#if only one person exists with given first name
		elif (matches == 1):
			c.execute("SELECT lastName FROM people WHERE firstName =?", [firstName])
			lastName = c.fetchone()[0]
			c.execute("DELETE FROM people WHERE firstName=? AND lastName=?", [firstName, lastName])
		elif (matches == 0):
			print ('Error: person with that name does not exist')

		if (c.rowcount == 1):
			print ('Successfully deleted ' + str(firstName) + " " + str(lastName))

	# if (c.rowcount == 1):
	# 	print ('Successfully deleted ' + str(firstName) + " " + str(lastName))
	# # elif (c.rowcount > 1):
	# # 	print(c.fetchone())
	# else:
	# 	print ('Could not delete ' + str(firstName)) # + " " + str(lastName)
	conn.commit()
	conn.close()

def readEmail(firstName, lastName=None):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	if (lastName):
		c.execute("SELECT email FROM people WHERE firstName =? AND lastName = ?", [firstName, lastName])
	else:
		c.execute("SELECT email FROM people WHERE firstName =?", [firstName])
	return (c.fetchone())
	#if multiple, specify ask which one
	conn.close()

def readName(email):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT firstName, lastName FROM people WHERE email =?", [email])
	name = c.fetchone()
	return name
	conn.close()	

def readLastName(firstName):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT lastName FROM people WHERE firstName =?", [firstName])
	lastName = c.fetchone()[0]
	return lastName
	conn.close()

def addEmail(email, firstName, lastName):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.executemany("UPDATE people SET email=? WHERE firstName=? AND lastName =?", [(email, firstName, lastName)])
	# c.executemany("INSERT INTO people (email) VALUES ? WHERE firstName = ?", [email, firstName])
	if (c.rowcount != 0):
		print ('Successfully added ' + email + ' to ' + firstName + " " + lastName)
	else:
		print ('Failed to add ' + email + ' to ' + firstName + " " + lastName)

		#add already exists if statement error message
	conn.commit()
	conn.close()

def countMatches(firstName):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT firstName, lastName FROM people WHERE firstName =?", [firstName])
	people = c.fetchall()
	return len(people)

def exit():
	sys.exit()

def selectNumber():
	try:
		ID = int(raw_input('--> '))
		return (ID)
	except ValueError:
		sys.exit("Error: expected an integer")

def selectBoolean():
	boolean = raw_input('--> ').capitalize()
	if (boolean == 'Y'):
		return True
		print('Y' + boolean)
	if (boolean == 'N'):
		return False
		print('N' + boolean)
	else:
		sys.exit("Error: expected boolean Y or N")

def importCSV():
	import csv

	conn = sqlite3.connect(db_path)
	c = conn.cursor()

	print('\nType path to file containing contacts \nFile must be in CSV format with data: full name, email, type (only name is required)')
	path = str(raw_input('--> '))
	if (os.path.exists(path)):
		with open(path, 'rb') as csvfile:
			contacts = csv.reader(csvfile, delimiter=',')
			for row in contacts:
				firstName, lastName = row[0].split()
				if (len(row) >= 2):
					email = row[1].strip()
				else:
					email = None
				if (len(row) == 3):
					contactType = row[2].strip()
				else:
					contactType = None
				c.execute("INSERT OR IGNORE into people (firstName, lastName, type, email) VALUES (?, ?, ?, ?)", [firstName, lastName, contactType, email])
				if (c.rowcount != 0):
					print ('Successfully added ' + firstName + ' ' + lastName)
				else:
					print ('Failed to add ' + firstName + ' ' + lastName)
				conn.commit()

		#add already exists if statement error message
	else:
		sys.exit('Path does not exist')

	conn.close()

#in case a query returns multiple results:
def selectPerson(firstName):
	conn = sqlite3.connect(db_path)
	c = conn.cursor()
	c.execute("SELECT firstName, lastName FROM people WHERE firstName =?", [firstName])
	people = c.fetchall()

	count = 0
	array = []
	print ("Multiple people found.  Select the correct person by ID number:")
	for person in people:
		print ("[" + str(count) + "]: " + str(person[0]) + " " + str(person[1]))
		array.append((count, person))
		count += 1		
	try:
		ID = int(raw_input('--> '))
		if (0 <= ID & ID <= (count - 1)):
			for person in array:
				if person[0] == ID:
					lastName = person[1][1]
					return lastName
		else:
			sys.exit("Error: integer not in ID range")
	except ValueError:
		sys.exit("Error: expected an integer")


from setuptools import find_packages, setup

dependencies = ['docopt']

setup(
    name='koto',
    version='0.2.0',
    url='https://github.com/shukmeister/koto',
    license='MIT',
    author='Ben Shukman',
    author_email='shukipost@gmail.com',
    description='Command line email tracking',
    long_description=__doc__,
    packages=find_packages(),
    package_data={'':[]},
    include_package_data=False,
    zip_safe=False,
    platforms='any',
    install_requires=dependencies,
    entry_points={
        'console_scripts': [
          'koto=koto.koto:main' #what dis? originally: 'find_email=find_email.find_email:main'
        ]
    },
    classifiers=[
        # As from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        # 'Development Status :: 1 - Planning',
          'Development Status :: 2 - Pre-Alpha',
        # 'Development Status :: 3 - Alpha',
        # 'Development Status :: 4 - Beta',
        # 'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)